/*************************************************************************************************************
/ IEC60870 RTU
/ Copyright by Panasonic Electric Works Europe AG 2003-2013
/ RT 17.07.2012
**************************************************************************************************************/

//Global variables for plc parameters
var m_MainConfiguration = "DT4400";	//Full PLC address of first database register
var m_MainConfigFactor = "DDT4402";	//Used part of the dut
var m_MainConfig104 = "DT4410";			//Used part of the dut
var m_MainConfigReset = "DT4411";		//Used part of the dut
var m_FTP_State = "DT800;"				//Full PLC address of first ftp status register
var m_FTP_StateDT;										//First ftp status address (DT area) calculated from m_FTP_State
var m_MainConfigurationDT;								//First Configuration address (DT area) calculated from m_MainConfiguration
var m_MainConfigFactorOffset;							//Offset to the variable in the configuration;
var m_MainConfig104Offset;								//Offset to the variable in the configuration;
var m_MainConfigResetOffset;							//Offset to the variable in the configuration;
var m_iSizeOfRecord = 20;								//m_MainConfiguration: point of data per record (use also number of 16bit values on 32bit values)
var m_iError = 0;											//Ajax communication error
var m_sINI_File = "";									//INI text file
var m_iModemEnabled;
var m_diLastASDU;											//To detect changes of the IOA

//System global 
var PLCreg = new Array(m_iSizeOfRecord);			//Response data array (16 bit words or integers)
var XML_Data;												//Buffer for MEWTOCOL request command
var PLC_StationAddress = "EE";						//PLC MEWTOCOL station address ("EE", "01"..."64")

//Global variables for displaying the plc data
var m_rFactor;
var m_i104;													//IEC60870-104 normal=0 modem=1 EM1=2 redundancy=3
var m_iResetStatus;										//Normally 0

//Global variables for interval handling
var m_bIsAreaDT;											//m_MainConfiguration in DT or FL area
var m_interval = 200;									//Main interval to update the data
var m_iSlowinterval = 2000;							//Slow interval to update the data during backup/restore
var m_iLoopInterval;										//Checks the transfer status and start the next download of data
var m_bDownloadOK;										//Handshake for data download
var m_iFirstTime;											//Work different in the first cycle
var m_iStatusFTP;											//Backup/Restore status
var m_bBackupRestore;									//Backup/Restore is active: slow update interval
var zeroes="00000";										//For generating leading zeroes in global.js

var m_DisplayHide = "none";
var m_DisplayView = "inline";
var m_TextDialRetries = "&nbsp;Dial retries (0=retry till connect)";
var m_TextPhone = "&nbsp;Phone number";
var m_TextBaud = "&nbsp;Baudrate";
var m_TextDataBits = "&nbsp;Data bits&nbsp;";
var m_TextParity = "&nbsp;Parity&nbsp;";
var m_TextFlow = "&nbsp;Flow control";
var m_TextMpWait = "&nbsp;Delay [ms]";
var m_TextMpLead = "&nbsp;Pre run [ms]";
var m_TextMpHold = "&nbsp;Post run [ms]";

var m_TextDataChanged = "<font color=\"#FF8040\">Changes not submitted!</font>";
var m_TextLoading = "<font color=\"#FF8040\">Getting data from RTU...</font>";
var m_TextComplete = "<font color=\"#1362AD\">RTU reading successful!</font>";
var m_TextNotComplete = "<font color=\"red\">Reload for reading data!</font>";
var m_TextCheckData = "<font color=\"red\">Data not transmitted. Please check the record parameters!</font>";
var m_TextRebootIt = "<font color=\"red\">Reboot required to use the new settings!</font>";
var m_TextAskToInitialize = "Type \"reset\" to set RTU default settings";
var m_TextInitialize = "<font color=\"red\">RTU settings will be reset...</font>";
var m_TextBackup = "<font color=\"#1362AD\">RTU settings backup in progress (up to 2min)...</font>";
var m_TextRestore = "<font color=\"#1362AD\">RTU settings restore in progress (up to 2min)...</font>";
var m_sTextErrorFTP = "<font color=\"red\">Backup/Restore error</font>";
var m_TextTransmissionError = "Message returned, but with error status.";
var m_TextErrorPost = "Submit post returned, but with error status.";
var m_sTextWaitForReset = "Rebooting in process. Please wait 15 seconds and then press enter.";
var m_TextAskToReboot = "Do you really want to reset the unit and to clear the telegram buffer!"
var m_TextAskToRestore = "Do you really want to restore a backup? All existing settings will be replaced!"
var GET_PLCDATA = 0;
var GET_PLCFTPSTATUS = 1;
var GET_IEC1 = 2;
var GET_CHIP = 3;
var GET_IEC2 = 4;

//JavaScript started on page load
window.onload = function()
{
	m_bDownloadOK = 1;								//Start download
	m_iLoopInterval = 0;								//Task manager
	m_bBackupRestore = false;						//Backup/Restore is not active

	//Preset the module number and start address: m_iModule is generated in the HTML page
	m_MainConfigurationDT = parseInt(m_MainConfiguration.substring(2));
	m_MainConfigFactorOffset = parseInt(m_MainConfigFactor.substring(3))-m_MainConfigurationDT; //DDT!!!
	m_MainConfigResetOffset = parseInt(m_MainConfigReset.substring(2))-m_MainConfigurationDT;
	m_MainConfig104Offset = parseInt(m_MainConfig104.substring(2))-m_MainConfigurationDT;
	m_bIsAreaDT = m_MainConfiguration.indexOf("DT");										// FL is -1
	m_FTP_StateDT = parseInt(m_FTP_State.substring(2));
	
	//debugger;
	$("ajaxStatus").innerHTML = m_TextLoading;
	
	intT = setInterval(function(){CheckIntervall();},m_interval);	//Set update interval
}


//Main function for data update
function CheckIntervall()
{
	$("ajaxDebug1").innerHTML = m_iLoopInterval;
	//$("ajaxDebug2").innerHTML = m_iLoopInterval;
	//$("ajaxDebug3").innerHTML = m_iError;
	//$("ajaxDebug4").innerHTML = m_iLoopInterval;

	if (m_bDownloadOK == 2)	//Reset or error
	{
		m_iLoopInterval = GET_PLCDATA;
		m_bDownloadOK = 1;
	}
	if (m_iLoopInterval == GET_PLCDATA && m_bDownloadOK == 1)
		XML_Data = GenerateRD(PLC_StationAddress, m_MainConfigurationDT, m_iSizeOfRecord, m_bIsAreaDT); //%01#RDD0019900204** --> XML_Data;
	if (m_iLoopInterval == GET_PLCFTPSTATUS && m_bDownloadOK == 1)
		XML_Data = GenerateRD(PLC_StationAddress, m_FTP_StateDT, 1, m_bIsAreaDT); //%01#RDD0019900204** --> XML_Data;
	if (m_iLoopInterval == GET_IEC1 && m_bDownloadOK == 1)
		XML_Data = "plc?mew60870";
	if (m_iLoopInterval == GET_CHIP && m_bDownloadOK == 1)
		XML_Data = "plc?chip";
	if (m_iLoopInterval == GET_IEC2 && m_bDownloadOK == 1)
		XML_Data = "plc?mew60871";
	
	if (m_bDownloadOK > 0)
	{
		m_bDownloadOK = 0;
		UpdateData(XML_Data);	// Request new data
	}

}


//This function checks the status of the xhr object and calls the corresponding routines to display the variables
function DisplayPLCdata()
{
	var iTemp;
	if(xhr.readyState == XHR_READYSTATE_COMPLETED)
	{
		if (xhr.status == XHR_STATUS_OK) {
			try
			{
				if (XML_Data.indexOf(".xml") > 0)
					plc = eval("(" + xhr.responseText + ")");			//Get PLC data
				else
				{
					if (m_iLoopInterval == GET_PLCDATA)
						decodeRD_BCD(xhr.responseText); //RD response string, true=integer false=word
					if (m_iLoopInterval == GET_PLCFTPSTATUS)
						decodeRD_BCD(xhr.responseText); //RD response string, true=integer false=word
					if (m_iLoopInterval == GET_IEC1)
						CheckIniFile(xhr.responseText, "IEC870");		//Check for any ini file section
					if (m_iLoopInterval == GET_CHIP)
						CheckIniFile(xhr.responseText, "MEW");			//Check for any ini file section
					/*
					if (m_iLoopInterval == GET_IEC2)
					{
						if(xhr.responseText.indexOf("[IEC870]") == -1)
							m_bRedundancy = false;							//File not available
						else
						{
							CheckIniFile(xhr.responseText, "IEC870");	//Check for any ini file section
							m_bRedundancy = true;
						}
					}
					*/
				}
			}
			catch (e)
			{
				doError(e + "\nPLC or JSON read error.");
				return;
			}
			if (m_iLoopInterval == GET_PLCDATA)
			{
				m_rFactor = Convert2Float(PLCreg[m_MainConfigFactorOffset], PLCreg[m_MainConfigFactorOffset+1]);
				DisplayValue("ajaxFactor", m_rFactor, 1, 0, 1);
				m_iResetStatus = parseInt(PLCreg[m_MainConfigResetOffset], 16);
				m_i104 = parseInt(PLCreg[m_MainConfig104Offset], 16);
				$("ajax104").value = m_i104;
			}
			if (m_iLoopInterval == GET_PLCFTPSTATUS)
			{
				m_iStatusFTP = parseInt(PLCreg[0], 16);
				if (m_iStatusFTP == -1 || m_iStatusFTP > 10)
					$("ajaxStatusFTP").innerHTML = m_sTextErrorFTP;
				if (m_iStatusFTP == 0)
					$("ajaxStatusFTP").innerHTML = "";
			}
			if (m_iLoopInterval == GET_IEC1)
			{
				iTemp = Get_Ini_String("IEC870", "LINK_ADD_SIZE");
				$("ajaxLinklength").value = Get_Ini_String("IEC870", "LINK_ADD_SIZE");
				$("ajaxASDUlength").value = Get_Ini_String("IEC870", "ASDU_ADD_SIZE");
				$("ajaxIOAlength").value = Get_Ini_String("IEC870", "IOBJ_ADD_SIZE");
				$("ajaxOriginator").value = Get_Ini_String("IEC870", "ORIGIN_ADD");
				
				$("ajaxLink").value = Get_Ini_String("IEC870", "LINK_ADD");
				iTemp = Get_Ini_String("IEC870", "ASDU_ADD");
				$("ajaxASDU00").value = iTemp;
				m_diLastASDU = iTemp;
				CalculateASDU(2);
				
				m_iModemEnabled = Get_Ini_String("IEC870", "M_ENABLE");
				if (isNaN(m_iModemEnabled))
					m_iModemEnabled = 0;
				
				$("ajaxModem").value = m_iModemEnabled; //mask
				
				$("ajaxPort").value = Get_Ini_String("IEC870", "104_PORT");
				
				$("ajaxT1").value = parseInt(Get_Ini_String("IEC870", "104_T1")/1000);
				$("ajaxT2").value = parseInt(Get_Ini_String("IEC870", "104_T2")/1000);
				$("ajaxT3").value = parseInt(Get_Ini_String("IEC870", "104_T3")/1000);
				$("ajaxK").value = Get_Ini_String("IEC870", "104_K");
				$("ajaxW").value = Get_Ini_String("IEC870", "104_W");
				
				iTemp = Get_Ini_String("IEC870", "104_TEST");
				$("ajaxTestInStop").checked = (iTemp == 1 || iTemp == 3)?true:false;
				$("ajaxTestInStart").checked = (iTemp == 2 || iTemp == 3)?true:false;
				
				iTemp = Get_Ini_String("IEC870", "C2REQ_C1DTA");
				$("ajaxClass1OnClass2").checked = (iTemp == 1)?true:false;

				$("ajaxRetries").value = Get_Ini_String("IEC870", "M_RETRY");
				$("ajaxPhone1").value = Get_Ini_String("IEC870", "M_LZTEL1");
				$("ajaxPhone2").value = Get_Ini_String("IEC870", "M_LZTEL2");
				
				$("ajaxFlow").value = Get_Ini_String("IEC870", "IEC_FLOW");
				$("ajaxMpWait").value = Get_Ini_String("IEC870", "MP_WAIT");
				$("ajaxMpLead").value = Get_Ini_String("IEC870", "MP_LEAD");
				$("ajaxMpHold").value = Get_Ini_String("IEC870", "MP_HOLD");
			}
			if (m_iLoopInterval == GET_CHIP)
			{
				$("ajaxBaud").value = Get_Ini_String("MEW", "SEC_BPS");
				$("ajaxDataBits").value = Get_Ini_String("MEW", "SEC_DTA");
				$("ajaxParity").value = Get_Ini_String("MEW", "SEC_PAR");
			}
			
			ShowModemData(false);

			if (m_iLoopInterval < GET_CHIP)	//Successful completed?
			{
				if (m_iResetStatus == 0)	//Get the status until the ResetStatus is null
				{
					if (m_iLoopInterval != GET_PLCFTPSTATUS || m_iStatusFTP <= 0 || m_iStatusFTP > 10)
						m_iLoopInterval++;
				}
				m_bDownloadOK = 1;
			}
			else
			{
				$("ajaxStatus").innerHTML = m_TextComplete;
				if (m_bBackupRestore)
				{
					clearInterval(intT);
					m_bBackupRestore = false;
					intT = setInterval(function(){CheckIntervall();},m_interval);	//Normal update interval
				}
			}
		}
		else
		{
			m_iError = m_iError + 1; // read error
			if (m_iError > 5) {
				m_iError = 0; // error reset
				$("ajaxStatus").innerHTML = m_TextNotComplete;
				alert(m_TextTransmissionError);
			}
			m_bDownloadOK = 2;
		}//end else XHR_STATUS_OK
	}//end if XHR_READYSTATE_COMPLETED
}


//Show or hide the modem settings
function ShowModemData(bCheckChanges)
{
	var iModemEnabled = parseInt($("ajaxModem").value);
	var iFlow = parseInt($("ajaxFlow").value);
	
	if (iModemEnabled >= 0 && iModemEnabled < 4)
	{
		$("ajaxBaud").style.display = m_DisplayView;
		$("ajaxDataBits").style.display = m_DisplayView;
		$("ajaxParity").style.display = m_DisplayView;
		$("ajaxFlow").style.display = m_DisplayView;
		$("ajaxTextBaud").innerHTML = m_TextBaud;
		$("ajaxTextDataBits").innerHTML = m_TextDataBits;
		$("ajaxTextParity").innerHTML = m_TextParity;
		$("ajaxTextFlow").innerHTML = m_TextFlow;		
	}
	else
	{
		$("ajaxBaud").style.display = m_DisplayHide;
		$("ajaxDataBits").style.display = m_DisplayHide;
		$("ajaxParity").style.display = m_DisplayHide;
		$("ajaxFlow").style.display = m_DisplayHide;
		$("ajaxTextBaud").innerHTML = "";
		$("ajaxTextDataBits").innerHTML = "";
		$("ajaxTextParity").innerHTML = "";
		$("ajaxTextFlow").innerHTML = "";
	}
	if ((iModemEnabled >= 0 && iModemEnabled < 4 && iFlow >= 4) && m_i104 < 3)
	{
		$("ajaxMpWait").style.display = m_DisplayView;
		$("ajaxMpLead").style.display = m_DisplayView;
		$("ajaxMpHold").style.display = m_DisplayView;
		$("ajaxTextMpWait").innerHTML = m_TextMpWait;
		$("ajaxTextMpLead").innerHTML = m_TextMpLead;
		$("ajaxTextMpHold").innerHTML = m_TextMpHold;
	}
	else
	{
		$("ajaxMpWait").style.display = m_DisplayHide;
		$("ajaxMpLead").style.display = m_DisplayHide;
		$("ajaxMpHold").style.display = m_DisplayHide;
		$("ajaxTextMpWait").innerHTML = "";
		$("ajaxTextMpLead").innerHTML = "";
		$("ajaxTextMpHold").innerHTML = "";
	}
	
	if (iModemEnabled == 3)
	{
		$("ajaxRetries").style.display = m_DisplayView;
		$("ajaxPhone1").style.display = m_DisplayView;
		$("ajaxPhone2").style.display = m_DisplayView;
		$("ajaxTextRetries").innerHTML = m_TextDialRetries;
		$("ajaxTextPhone1").innerHTML = m_TextPhone + " 1";
		$("ajaxTextPhone2").innerHTML = m_TextPhone + " 2";
	}
	else
	{
		$("ajaxRetries").style.display = m_DisplayHide;
		$("ajaxPhone1").style.display = m_DisplayHide;
		$("ajaxPhone2").style.display = m_DisplayHide;
		$("ajaxTextRetries").innerHTML = "";
		$("ajaxTextPhone1").innerHTML = "";
		$("ajaxTextPhone2").innerHTML = "";
	}
	
	if (iModemEnabled > 0 && iModemEnabled < 4)
	{
		$("ajaxPort").disabled = true;
		$("ajaxT1").disabled = true;
		$("ajaxT2").disabled = true;
		$("ajaxT3").disabled = true;
		$("ajaxK").disabled = true;
		$("ajaxW").disabled = true;
		$("ajaxTestInStop").disabled = true;
		$("ajaxTestInStart").disabled = true;
	}
	else
	{
		$("ajaxPort").disabled = false;
		$("ajaxT1").disabled = false;
		$("ajaxT2").disabled = false;
		$("ajaxT3").disabled = false;
		$("ajaxK").disabled = false;
		$("ajaxW").disabled = false;
		$("ajaxTestInStop").disabled = false;
		$("ajaxTestInStart").disabled = false;
	}
	
	if (iModemEnabled != m_iModemEnabled && bCheckChanges == true)
		DetectChange();
	m_iModemEnabled = iModemEnabled;
}


//This function writes the values entered in the formular fields and assigns it to the correspong DTs/ini files
function SetData(iMode)
{
	var sTemp, iFlow;
	var iTemp1, iTemp2, iTemp3, iTemp4, iTemp5;
	var iError = 0;
	var sPostmessage, sPostmessageMEWIEC;
	if (iMode == 1)
	{
		//IEC60870 main parameter
		iTemp1 = parseInt($("ajaxLinklength").value);
		if (iTemp1 >= 0)
			sPostmessageMEWIEC = CreatePost_Ini_String("IEC870","LINK_ADD_SIZE", iTemp1);
		else
			iError++;
		if(iTemp1 == 2)
			sTemp = "65535";
		else
			sTemp = "255";
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","LINK_ADD_BC", sTemp); //Additional option without input field
		iTemp1 = parseInt($("ajaxASDUlength").value);
		if (iTemp1 >= 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","ASDU_ADD_SIZE", iTemp1);
		else
			iError++;
		iTemp1 = parseInt($("ajaxIOAlength").value);
		if (iTemp1 >= 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","IOBJ_ADD_SIZE", iTemp1);
		else
			iError++;
		
		iTemp1 = parseInt($("ajaxLink").value);
		if (iTemp1 > 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","LINK_ADD", iTemp1);
		else
			iError++;
			
		iTemp1 = parseInt($("ajaxASDU00").value);
		if (iTemp1 > 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","ASDU_ADD", iTemp1);
		else
			iError++;
		iTemp1 = parseInt($("ajaxOriginator").value);
		if (iTemp1 > 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","ORIGIN_ADD", iTemp1);
		else
			iError++;
		
		iTemp1 = parseInt($("ajaxPort").value);
		if (iTemp1 > 1025 && iTemp1 < 65536)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_PORT", iTemp1);
		else
			iError++;
		
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","M_ENABLE", m_iModemEnabled);
		
		//IEC60870 timing
		iTemp1 = parseInt($("ajaxT1").value);
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_T1", iTemp1*1000);
		iTemp2 = parseInt($("ajaxT2").value);
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_T2", iTemp2*1000);
		iTemp3 = parseInt($("ajaxT3").value);
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_T3", iTemp3*1000);
		iTemp4 = parseInt($("ajaxK").value);
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_K", iTemp4);
		iTemp5 = parseInt($("ajaxW").value);
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_W", iTemp5);
		if (iTemp1 < iTemp3 && iTemp2 < iTemp1 && iTemp5 < iTemp4 && iTemp4 < 15)
			iTemp1 = 0; //Dummy, OK
		else
			iError++;
		
		iTemp1 = ($("ajaxTestInStop").checked == true)?1:0;		//Combined!!!
		iTemp1 += ($("ajaxTestInStart").checked == true)?2:0;	//Combined!!!
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","104_TEST", iTemp1);
		
		iTemp1 = ($("ajaxClass1OnClass2").checked == true)?1:0;
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","C2REQ_C1DTA", iTemp1);

		//Modem dial out
		iTemp1 = parseInt($("ajaxRetries").value);
		if (iTemp1 >= 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","M_RETRY", iTemp1);
		else
			iError++;
		sTemp = $("ajaxPhone1").value;
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","M_LZTEL1", sTemp);
		sTemp = $("ajaxPhone2").value;
		sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","M_LZTEL2", sTemp);
		
		//PLC parameter
		m_rFactor = $("ajaxFactor").value;

		sPostmessage = CreatePost_REAL_DDT(m_MainConfigurationDT+m_MainConfigFactorOffset, m_rFactor);
		sPostmessage += "&" + CreatePost_DT(m_MainConfigurationDT+m_MainConfig104Offset,m_i104);
		
		//CHIP parameter
		iTemp1 = parseInt($("ajaxBaud").value);
		if (iTemp1 >= 300)
		{
			sPostmessage += "&" + CreatePost_Ini_String("MEW","SEC_BPS", iTemp1);
			iTemp1 = 99000 / iTemp1;	//Time in ms for 9 characters (each 11bit)
			if(iTemp1 < 10)
				iTemp1 = 10;
			sPostmessage += "&" + CreatePost_Ini_String("IEC870","IEC_ICTO", iTemp1); //Additional option without input field
		}
		else
			iError++;
		iTemp1 = parseInt($("ajaxDataBits").value);
		if (iTemp1 >= 7)
			sPostmessage += "&" + CreatePost_Ini_String("MEW","SEC_DTA", iTemp1);
		else
			iError++;
		iTemp1 = parseInt($("ajaxParity").value);
		if (iTemp1 >= 0)
			sPostmessage += "&" + CreatePost_Ini_String("MEW","SEC_PAR", iTemp1);
		else
			iError++;
		iFlow = parseInt($("ajaxFlow").value);
		if (iFlow >= 0)
			sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","IEC_FLOW", iFlow);
		else
			iError++;
		if(iFlow >=4)
		{
			iTemp1 = parseInt($("ajaxMpWait").value);
			if (iTemp1 >= 10 && iTemp1 < 999)
				sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","MP_WAIT", iTemp1);
			else
				iError++;
			iTemp1 = parseInt($("ajaxMpLead").value);
			if (iTemp1 >= 10 && iTemp1 < 999)
				sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","MP_LEAD", iTemp1);
			else
				iError++;
			iTemp1 = parseInt($("ajaxMpHold").value);
			if (iTemp1 >= 10 && iTemp1 < 999)
				sPostmessageMEWIEC += "&" + CreatePost_Ini_String("IEC870","MP_HOLD", iTemp1);
			else
				iError++;
		}
		$("ajaxStatus").innerHTML = m_TextLoading;
		sPostmessage += "&" + sPostmessageMEWIEC;	//Complete settings
		
		if (iError == 0)
		{
			PostNow(sPostmessage);	//The redundany parameter will be duplicated by the driver internally
			$("ajaxReboot").innerHTML = m_TextRebootIt;
			m_bDownloadOK = 2;
		}
		else
		{
			$("ajaxStatus").innerHTML = m_TextCheckData;
		}
	}
}


//This function writes the values entered in the formular fields and assigns it to the correspong DTs/ini files
function RebootUnit(iMode)
{
	if (iMode == 1)
	{
		var bCheck = confirm(m_TextAskToReboot);
		if (bCheck == true)
		{
			SendDT(m_MainConfigurationDT+m_MainConfigResetOffset, 870);
			alert(m_sTextWaitForReset);
			m_bDownloadOK = 2;
			$("ajaxStatus").innerHTML = m_TextCheckData;
			$("ajaxReboot").innerHTML = "";
		}
	}
}


//This function writes the values entered in the formular fields and assigns it to the correspong DTs/ini files
function InitializeUnit(iMode)
{
	if (iMode == 1)
	{
		var sCheck = prompt(m_TextAskToInitialize, "");
		if (sCheck == "reset")
		{
			SendDT(m_MainConfigurationDT+m_MainConfigResetOffset, 6870);
			m_bDownloadOK = 2;
			$("ajaxStatus").innerHTML = m_TextInitialize;
		}	
	}
}


//This function writes the values entered in the formular fields and assigns it to the correspong DTs/ini files
function BackupUnit(iMode)
{
	if (iMode == 1)
	{
		SendDT(m_MainConfigurationDT+m_MainConfigResetOffset, 111);
		$("ajaxStatus").innerHTML = m_TextBackup;
		clearInterval(intT);
		m_bDownloadOK = 2;//Restart communication
		m_bBackupRestore = true;
		intT = setInterval(function(){CheckIntervall();},m_iSlowinterval);	//Set slow update interval
	}
}


//This function writes the values entered in the formular fields and assigns it to the correspong DTs/ini files
function RestoreUnit(iMode)
{
	if (iMode == 1)
	{
		var bCheck = confirm(m_TextAskToRestore);
		if (bCheck == true)
		{
			SendDT(m_MainConfigurationDT+m_MainConfigResetOffset, 222);
			$("ajaxStatus").innerHTML = m_TextRestore;
			clearInterval(intT);
			m_bDownloadOK = 2;//Restart communication
			m_bBackupRestore = true;
			intT = setInterval(function(){CheckIntervall();},m_iSlowinterval);	//Set slow update interval
		}
	}
}


//This function calculates the ASDU in different ways: a single integer value or a value split into three parts (bytes)
function CalculateASDU(iMode)
{
	var bError = false;
	var iASDU = $("ajaxASDU00").value;
	var iASDU1 = parseInt($("ajaxASDU01").value,10);
	var iASDU2 = parseInt($("ajaxASDU02").value,10);
	
	if (iMode == 1)
	{
		if(iASDU1 < 256 && iASDU2 < 256 && (iASDU1+iASDU2 > 0))
		{
			iASDU = (iASDU2 * 256) + iASDU1;
			$("ajaxASDU00").value = iASDU;
			if(m_diLastASDU != iASDU)
				DetectChange();
			m_diLastASDU = iASDU;
		}
		else
			bError = true;
	}
	if (iMode == 2)
	{
		if (iASDU != "" && iASDU > 0)
		{
			if(m_diLastASDU != iASDU);
			m_diLastASDU = iASDU;
			iASDU2 = Math.floor(iASDU / 256);
			iASDU1 = iASDU - iASDU2*256;
			$("ajaxASDU01").value = iASDU1;
			$("ajaxASDU02").value = iASDU2;
		}
		else
			bError = true;
	}
	if (bError == true)
	{
		if (m_diLastASDU <= 0 || isNaN(m_diLastASDU))
			m_diLastASDU = 1;
		$("ajaxASDU00").value = m_diLastASDU;
		CalculateASDU(2);
	}
}


//Notify message
function DetectChange()
{
	$("ajaxStatus").innerHTML = m_TextDataChanged;
}


//MEWTOCOL data must be converted from hex byte fragments to a complete FLOAT value
function Convert2Float(PLCreg0, PLCreg1)
{
	var sStatus = "";
	var iStatus;
	ieee32 = new ieee();	//REAL/FLOAT converting
	sStatus = PLCreg1.substring(0, 2);
	sStatus = sStatus + PLCreg1.substring(2, 4);
	sStatus = sStatus + PLCreg0.substring(0, 2);
	sStatus = sStatus + PLCreg0.substring(2, 4);
	if (sStatus != "00000000")
	{
		ieee32.BinString = ieee32.Hex2Bin(sStatus);
		iStatus = ieee32.Convert2Dec();
		return iStatus; //.toPrecision(6);
	}
	else
		return 0;
}


//Submit a message with values to the PLC
function PostNow(sPostMessage)
{
	xhr_SendDT = createXMLHttpRequest();

	xhr_SendDT.onreadystatechange = function()		// Details see "C.) Programming Examples..."
	{
		if (xhr_SendDT.readyState == XHR_READYSTATE_COMPLETED)
	{
			if (xhr_SendDT.status != XHR_STATUS_OK)	//Check http answer
				doError(m_TextErrorPost);
		}
	};

	xhr_SendDT.open("POST","/plcpost",true);			//Open a http POST connect
	xhr_SendDT.send(sPostMessage);						// and submit CGI parameter
}


//Prepare submit message for chip.ini parameters
//CreatePost_Chip(9999,"[IEC870]M_LZTELx:12345");
function CreatePost_Ini_String(sSection, sAddress, sValue)
{
	return "{CH_999_9_770_%i}=[" + sSection + "]" + sAddress + ":" + sValue;
}


//Prepare submit message for 16 bit integer value to PLC DT address
function CreatePost_DT(address, value)
{
	var add = "0" + address;								// Generate PLC submit string,
	var adl = add.length;									// for example: DT123 <-- 456
	var par = "{DT_" + add.substring(0,adl-1) + "_" + add.substring(adl-1,adl);
	par = par + "_770_%i}=" + value;	// result: "{DT_12_3_770_%i}=456"
	return par;
}


//Prepare submit message for  32 bit unsigned integer value to PLC DDT address
function CreatePost_UI_DDT(address, value)
{
	var add = "0" + address;								// Generate PLC submit string,
	var adl = add.length;									// for example: DT123 <-- 456
	var par = "{DT_" + add.substring(0,adl-1) + "_" + add.substring(adl-1,adl);
	par = par + "_1028_%lu}=" + value;					// result: "{DT_12_3_1028_%lu}=456"
	return par;
}


//Prepare submit message for 32 bit floating point value to PLC DDT address
function CreatePost_REAL_DDT(address, value)
{
	var add = "0" + address;								// Generate PLC submit string,
	var adl = add.length;									// for example: DT123 <-- 456
	var par = "{DT_" + add.substring(0,adl-1) + "_" + add.substring(adl-1,adl);
	par = par + "_2052_%f}=" + value;					// result: "{DT_12_3_2052_%f}=456.7"
	return par;
}


//Validate the ini file
function CheckIniFile(sResponseText, sSection)
{
	if(sResponseText.indexOf("["+sSection+"]") == -1)
	{
		doError("INI file read error.");
		return;
	}
	m_sINI_File = sResponseText;
}


//Get a ini file setting item
function Get_Ini_String(sSection, sItem)
{
	var sItemText = "";
	var pSectionStart = m_sINI_File.indexOf("[" + sSection + "]") + sSection.length + 2;
	if(pSectionStart >= 0)
	{
		
		var pItemEnd = m_sINI_File.indexOf("\n"+sItem+"=", pSectionStart);//+sItem.length+1;
		var pSectionEnd = m_sINI_File.indexOf("[", pSectionStart);
		if (pItemEnd != -1 && (pItemEnd < pSectionEnd || pSectionEnd == -1))
		{
			pItemEnd += sItem.length+2; //Add length for the chars '\n='
			sItemText = m_sINI_File.substring(pItemEnd,pItemEnd+40);
			var pItemTextEnd = sItemText.search(/[\s\/#]/); //Find whitespace characters and comment
			if (pItemTextEnd != -1)
				sItemText = m_sINI_File.substring(pItemEnd,pItemEnd+pItemTextEnd);
		}
	}
	return sItemText;
}
/* End of file */